@@ -14,6 +14,6 @@ my $client = Mojo::Weixin->new(
     (defined $ENV{QRCODE_PATH}?(qrcode_path =>  $ENV{QRCODE_PATH}):()),
 );
 $client->load("ShowMsg");
-$client->load("Openwx",data=>{listen=>[{host=>$host,port=>$port}], post_api=>$post_api});
+$client->load("Openwx",data=>{listen=>[{host=>$host,port=>$port}], post_api=>$post_api,post_event=>1});
 $client->load("UploadQRcode");
 $client->run();
